﻿/**************************************************************************
@ file name ClickableMapPanel.jsx
@ description HTML用のクリッカブルマップのコードを生成するパネル
@ version 1.0
@ author Tsudoi Yoshimoto
@ url http://tsudoi.org/weblog/?p=1279
@ last update date 2013.05.03
**************************************************************************/

/*
* description : パネルからのクリック通知
* param _type(string) : shape type 'rect','circle','poly','path'
*/
function getClickableMap(_type){	
	
	var _pathItems = app.activeDocument.pathItems;
	var _pointObj;
	var _bounds;
	var _xml = "<object>"; 
	
	var _color = { 'r':102, 'g':255, 'b':255 };
	var _opacity = 75;
	
	//パスレイヤーを作成
	if(
		_type == 'rect'
		|| _type == 'poly'
	){
		addPathLayer();
	}
	
	//チェック
	switch(_type){
		case 'rect':
			_pointObj = getRectPoints(_pathItems[0].subPathItems[0].pathPoints);
			if( isSquare( _pointObj) ){
				
			}
			else{
				removePathLayer();
				_xml += convertToXML('', "coords");
				_xml += convertToXML("rectは選択範囲が正方形である必要があります。", "error");
				_xml += "</object>";
				return _xml;
			}
			break;
		case 'circle':
			_bounds = app.activeDocument.selection.bounds;
			_pointObj = { 'x0':parseInt(_bounds[0]), 'y0':parseInt(_bounds[1]), 'x1':parseInt(_bounds[2]), 'y1':parseInt(_bounds[3]) };
			if( isSquare(_pointObj)  ){

			}
			else{
					_xml += convertToXML('', "coords");
					_xml += convertToXML("circleは選択範囲が正円である必要があります。", "error");
					_xml += "</object>";
					return _xml;
				}
			break;
		case 'poly':

			break;
		case 'path':

			break;
	}
	
	//座標を取得
	var _coords = '';
	switch(_type){
		case 'rect':
			_coords = getRectCoords( _pointObj );
			break;
		case 'circle':
			_coords = getCircleCoords( _bounds );
			break;
		case 'poly':
		case 'path':
			_coords = getPolyCoords( _pathItems[0].subPathItems[0].pathPoints );
			break;
	}	
	
	//shapeレイヤーの作成
	switch(_type){
		case 'rect':
			addRectShapeLayer(_pointObj, _color, _opacity, 'rect_map');
			break;
		case 'circle':
			addCircleShapeLayer(_pointObj, _color, _opacity, 'circle_map');
			break;
		case 'poly':
			addPolyShapeLayer(_color, _opacity, 'poly_map');
			break;
		case 'path':
			addPathShapeLayer(_color, _opacity, 'path_map');
			break;
	}

	//パスレイヤーを削除
	if(
		_type == 'rect'
		|| _type == 'poly'
	){
		removePathLayer();
	}
	
	_xml += convertToXML(_coords, "coords");
	_xml += "</object>";
	
	return _xml;
	
}

/*
* description : パスレイヤーの作成
*/
function addPathLayer(){
	var idMk = charIDToTypeID( "Mk  " );
	var desc40 = new ActionDescriptor();
	var idnull = charIDToTypeID( "null" );
	var ref35 = new ActionReference();
	var idPath = charIDToTypeID( "Path" );
	ref35.putClass( idPath );
	desc40.putReference( idnull, ref35 );
	var idFrom = charIDToTypeID( "From" );
	var ref36 = new ActionReference();
	var idcsel = charIDToTypeID( "csel" );
	var idfsel = charIDToTypeID( "fsel" );
	ref36.putProperty( idcsel, idfsel );
	desc40.putReference( idFrom, ref36 );
	var idTlrn = charIDToTypeID( "Tlrn" );
	var idPxl = charIDToTypeID( "#Pxl" );
	desc40.putUnitDouble( idTlrn, idPxl, 2.000000 );
	executeAction( idMk, desc40, DialogModes.NO );
}

/*
* description : パスレイヤーを削除
*/
function removePathLayer(){
	var idDlt = charIDToTypeID( "Dlt " );
	var desc4 = new ActionDescriptor();
	var idnull = charIDToTypeID( "null" );
	var ref2 = new ActionReference();
	var idPath = charIDToTypeID( "Path" );
	var idWrPt = charIDToTypeID( "WrPt" );
	ref2.putProperty( idPath, idWrPt );
	desc4.putReference( idnull, ref2 );
	executeAction( idDlt, desc4, DialogModes.NO );
}

/*
* description : rectのレイヤー生成
* param _pointObj(object) : 座標を格納したオブジェクト
* param _color(object) : カラーを格納したオブジェクト
* param _opacity(number) : 透明度（0~100）
* param _name(string) : Layer名
*/
function addRectShapeLayer(_pointObj,_color,_opacity, _name){
	
	//Shapeを描画し、レイヤーを作成
	var _idMk = charIDToTypeID( "Mk  " );
	var _desc = getNewLayer();
	
		var _idUsng = charIDToTypeID( "Usng" );
		var _desc2 = new ActionDescriptor();
			//Shapeのカラーを設定する
			setShapeColor(_desc2,_color);
			//Shapeの描画
			drawShape( _desc2, _pointObj, "Rctn" );
			//Layerの透明度を設定する
			setLayerOpacity(_desc2, _opacity);
		var _idcontentLayer = stringIDToTypeID( "contentLayer" );
		_desc.putObject( _idUsng, _idcontentLayer, _desc2 );

	executeAction( _idMk, _desc, DialogModes.NO );	
	
	//作成したレイヤーにレイヤー名を設定
	var _idsetd = charIDToTypeID( "setd" );
	var _desc3 = new ActionDescriptor();

		//Layer名を変更する
		setLayerName(_desc3, _name);

	executeAction( _idsetd, _desc3, DialogModes.NO );

}

/*
* description : circleのレイヤー生成
* param _pointObj(object) : 座標を格納したオブジェクト
* param _color(object) : カラーを格納したオブジェクト
* param _opacity(number) : 透明度（0~100）
* param _name(string) : Layer名
*/
function addCircleShapeLayer(_pointObj,_color,_opacity, _name){
	
	//Shapeを描画し、レイヤーを作成
	var _idMk = charIDToTypeID( "Mk  " );
	var _desc = getNewLayer();
	
		var _idUsng = charIDToTypeID( "Usng" );
		var _desc2 = new ActionDescriptor();
			
			//Shapeのカラーを設定する
			setShapeColor(_desc2,_color);
			
			//Shapeの描画
			drawShape( _desc2, _pointObj, "Elps" );
			
			//Layerの透明度を設定する
			setLayerOpacity(_desc2, _opacity);
			
		var _idcontentLayer = stringIDToTypeID( "contentLayer" );
		_desc.putObject( _idUsng, _idcontentLayer, _desc2 );
	
	executeAction( _idMk, _desc, DialogModes.NO );
	
	//作成したレイヤーにレイヤー名を設定
	var _idsetd = charIDToTypeID( "setd" );
	var _desc3 = new ActionDescriptor();

		//Layer名を変更する
		setLayerName(_desc3, _name);

	executeAction( _idsetd, _desc3, DialogModes.NO );
	
}

/*
* description : polyのレイヤー生成
* param _pointObj(object) : 座標を格納したオブジェクト
* param _color(object) : カラーを格納したオブジェクト
* param _opacity(number) : 透明度（0~100）
* param _name(string) : Layer名
*/
function addPolyShapeLayer(_color,_opacity, _name){

	//Shapeを描画し、レイヤーを作成
	var _idMk = charIDToTypeID( "Mk  " );
	var _desc = getNewLayer();
	
		var _idUsng = charIDToTypeID( "Usng" );
		var _desc2 = new ActionDescriptor();
			
			//Shapeのカラーを設定する
			setShapeColor(_desc2,_color);
			
			//Shapeの描画
			drawPath( _desc2);
			
			//Layerの透明度を設定する
			setLayerOpacity(_desc2, _opacity);
			
		var _idcontentLayer = stringIDToTypeID( "contentLayer" );
		_desc.putObject( _idUsng, _idcontentLayer, _desc2 );
	
	executeAction( _idMk, _desc, DialogModes.NO );

	//作成したレイヤーにレイヤー名を設定
	var _idsetd = charIDToTypeID( "setd" );
	var _desc3 = new ActionDescriptor();

		//Layer名を変更する
		setLayerName(_desc3, _name);

	executeAction( _idsetd, _desc3, DialogModes.NO );
	
}

/*
* description : pathのレイヤー生成
* param _pointObj(object) : 座標を格納したオブジェクト
* param _color(object) : カラーを格納したオブジェクト
* param _opacity(number) : 透明度（0~100）
* param _name(string) : Layer名
*/
function addPathShapeLayer(_color,_opacity, _name){
	selectionFromPath();
	addPolyShapeLayer(_color,_opacity, _name);
}

/*
* description : パスから選択範囲に変更
*/
function selectionFromPath(){
var idsetd = charIDToTypeID( "setd" );
    var desc34 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref14 = new ActionReference();
        var idChnl = charIDToTypeID( "Chnl" );
        var idfsel = charIDToTypeID( "fsel" );
        ref14.putProperty( idChnl, idfsel );
    desc34.putReference( idnull, ref14 );
    var idT = charIDToTypeID( "T   " );
        var ref15 = new ActionReference();
        var idPath = charIDToTypeID( "Path" );
        var idPath = charIDToTypeID( "Path" );
        var idvectorMask = stringIDToTypeID( "vectorMask" );
        ref15.putEnumerated( idPath, idPath, idvectorMask );
        var idLyr = charIDToTypeID( "Lyr " );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref15.putEnumerated( idLyr, idOrdn, idTrgt );
    desc34.putReference( idT, ref15 );
    var idVrsn = charIDToTypeID( "Vrsn" );
    desc34.putInteger( idVrsn, 1 );
    var idvectorMaskParams = stringIDToTypeID( "vectorMaskParams" );
    desc34.putBoolean( idvectorMaskParams, true );
executeAction( idsetd, desc34, DialogModes.NO );
}

/*
* description : 新規Layerを取得
* return ActionDescriptor;
*/
function getNewLayer(){
	var _desc = new ActionDescriptor();
	var _idnull = charIDToTypeID( "null" );
	var _ref = new ActionReference();
		var _idcontentLayer = stringIDToTypeID( "contentLayer" );
		_ref.putClass( _idcontentLayer );
	_desc.putReference( _idnull, _ref );
	return _desc;
}

/*
* description : Layer名を変更する
* param _desc(ActionDescriptor) : 対象ActionDescriptor
* param _name(string) : Layer名
*/
function setLayerName(_desc,_name){

	var _idnull = charIDToTypeID( "null" );
	var _ref2 = new ActionReference();
		var _idLyr = charIDToTypeID( "Lyr " );
		var _idOrdn = charIDToTypeID( "Ordn" );
		var _idTrgt = charIDToTypeID( "Trgt" );
		_ref2.putEnumerated( _idLyr, _idOrdn, _idTrgt );
	_desc.putReference( _idnull, _ref2 );
	
    var _idT = charIDToTypeID( "T   " );

        var _desc2 = new ActionDescriptor();
        var _idNm = charIDToTypeID( "Nm  " );
        _desc2.putString( _idNm, _name);

    var _idLyr = charIDToTypeID( "Lyr " );
    _desc.putObject( _idT, _idLyr, _desc2 );
	
}

/*
* description : Shapeの描画
* param _desc(ActionDescriptor) : 対象ActionDescriptor
* param _pointObj(object) : 座標を格納したオブジェクト
* param _type(object) : shapeタイプ（ 'Rctn', 'Elps' ）
*/
function drawShape(_desc,_pointObj, _type){
	var _idShp = charIDToTypeID( "Shp " );
		var _desc2 = new ActionDescriptor();
		
		//top
		var _idTop = charIDToTypeID( "Top " );
		var _idPxl = charIDToTypeID( "#Pxl" );
		_desc2.putUnitDouble( _idTop, _idPxl, _pointObj.y0 );
		
		//left
		var _idLeft = charIDToTypeID( "Left" );
		var _idPxl = charIDToTypeID( "#Pxl" );
		_desc2.putUnitDouble( _idLeft, _idPxl, _pointObj.x0 );
		
		//bottom
		var _idBtom = charIDToTypeID( "Btom" );
		var _idPxl = charIDToTypeID( "#Pxl" );
		_desc2.putUnitDouble( _idBtom, _idPxl, _pointObj.y1 );
		
		//right
		var _idRght = charIDToTypeID( "Rght" );
		var _idPxl = charIDToTypeID( "#Pxl" );
		_desc2.putUnitDouble( _idRght, _idPxl, _pointObj.x1 );
		
	var _idRctn = charIDToTypeID( _type );
	_desc.putObject( _idShp, _idRctn, _desc2 );
}

/*
* description : Pathを塗りつぶす
* param _desc(ActionDescriptor) : 対象ActionDescriptor
*/
function drawPath(_desc){
	var _idnull = charIDToTypeID( "null" );
	var _ref = new ActionReference();
	var _idPath = charIDToTypeID( "Path" );
	var _idWrPt = charIDToTypeID( "WrPt" );
	_ref.putProperty( _idPath, _idWrPt );
	_desc.putReference( _idnull, _ref );
	var _idWhPt = charIDToTypeID( "WhPt" );
	_desc.putBoolean( _idWhPt, true );
}

/*
* description : Shapeのカラーを設定する
* param _desc(ActionDescriptor) : 対象ActionDescriptor
* param _color(object) : カラーを格納したオブジェクト
*/
function setShapeColor(_desc,_color){
	var _idType = charIDToTypeID( "Type" );
	
		var _desc2 = new ActionDescriptor();
		var _idClr = charIDToTypeID( "Clr " );
		
			var _desc3 = new ActionDescriptor();
			
			//red
			var _idRd = charIDToTypeID( "Rd  " );
			_desc3.putDouble( _idRd, _color.r );
			
			//green
			var _idGrn = charIDToTypeID( "Grn " );
			_desc3.putDouble( _idGrn, _color.g );
			
			//blue
			var _idBl = charIDToTypeID( "Bl  " );
			_desc3.putDouble( _idBl, _color.b );
			
		var _idRGBC = charIDToTypeID( "RGBC" );
		_desc2.putObject( _idClr, _idRGBC, _desc3 );
	
	var _idsolidColorLayer = stringIDToTypeID( "solidColorLayer" );
	_desc.putObject( _idType, _idsolidColorLayer, _desc2 );
}

/*
* description : Layerの透明度を設定する
* param _desc(ActionDescriptor) : 対象ActionDescriptor
* param _opacity(number) : 透明度（0~100）
*/
function setLayerOpacity(_desc,_opacity){
	var _idOpct = charIDToTypeID( "Opct" );
	var _idPrc = charIDToTypeID( "#Prc" );
	_desc.putUnitDouble( _idOpct, _idPrc, _opacity );
}

/*
* description : shape type : rect のコード取得
* param _pointObj(object) : 座標を格納したオブジェクト
* return string;
*/
function getRectCoords(_pointObj){
	var _coords = '';
	_coords = _pointObj.x0+','+_pointObj.y0+','+_pointObj.x1+','+_pointObj.y1;
	return _coords;
}

/*
* description : shape type : rect 時の始点と終点の座標を取得する
* param _points(array) : 選択範囲のポイント配列
* return object;
*/
function getRectPoints(_points){
	var _anchor;
	var _maxX = 0;
	var _minX = 0;
	var _maxY = 0;
	var _minY = 0;
	for( var i=0; i<_points.length; i++ ){
		_anchor = _points[i].anchor;
		if( i == 0 ){
			_maxX = _anchor[0];
			_minX = _anchor[0];
			_maxY = _anchor[1];
			_minY = _anchor[1];
		}
		else{
			_maxX = Math.max( _anchor[0], _maxX );
			_maxY = Math.max( _anchor[1], _maxY );
			_minX = Math.min( _anchor[0], _minX );
			_minY = Math.min( _anchor[1], _minY );
		}
	}
	return { 'x0':_minX, 'y0':_minY, 'x1':_maxX, 'y1':_maxY };
}

/*
* description : shape type : circle のコード取得
* param _points(array) : 選択範囲のポイント配列
* return string;
*/
function getCircleCoords(_bounds){
	var _x0 = parseInt(_bounds[0]);
	var _y0 = parseInt(_bounds[1]);
	var _x1 = parseInt(_bounds[2]);
	var _y1 = parseInt(_bounds[3]);
	var _radius = (_x1-_x0)/2;
	_coords = (_x0+_radius)+','+(_y0+_radius)+','+_radius;
	return _coords;
}

/*
* description : shape type : poly のコード取得
* param _points(array) : 選択範囲のポイント配列
* return string;
*/
function getPolyCoords(_points){
	var _coords = '';
	for( var i=0; i<_points.length; i++ ){
		_coords += Math.floor(_points[i].anchor[0])+',';
		_coords += Math.floor(_points[i].anchor[1])+',';
	}
	_coords = _coords.slice(0,_coords.length-1);
	return _coords;
}

/*
* description : 正方形かどうか
* param _pointObj(object) : 座標を格納したオブジェクト
* return boolean;
*/
function isSquare(_pointObj){
	var _flag = true;
	//if( _points.length == 4 ){
		var _p0 = _pointObj.x1 - _pointObj.x0;
		var _p1 = _pointObj.y1 - _pointObj.y0;
		if( _p0 == _p1 ){
			_flag = true;
		}
		else{
			_flag = false;
		}
	//}
	//else{
	//	_flag = false;
	//}
	return _flag;
}

function convertToXML(property, identifier){
	var type = typeof property;
	var xml = '<property id = "' + identifier + '" >';
	
	switch(type){
		case "number":
			xml += "<number>";
			xml += property.toString();
			xml += "</number>";
			break;
		case "boolean":
			xml += "<" + property.toString() + "/>";
			break;
		case "string":
			xml += "<string>";
			xml += property.toString();
			xml += "</string>";
			break;
		case "object":
			alertDebug("Object case is currently not supported");
			break;
		case "undefined":
			xml += "<string>undefined</string>";
			break;
		default:
			alertDebug("Type " + type + " is unknown.");
			return "";
	}
	xml += '</property>';
	return xml;
}